@@ -1,4 +1,4 @@
-(* $Id: simple.ml,v 1.5.6.1 2004/03/11 01:19:43 garrigue Exp $ *)
+(* $Id: simple.ml,v 1.6 2003/02/20 06:47:48 garrigue Exp $ *)
 
 open GMain
 
@@ -6,7 +6,8 @@ let main () =
   let w = GWindow.window ~title:"LablGL/Gtk" () in
   w#connect#destroy ~callback:Main.quit;
   let area =
-    GlGtk.area [`RGBA;`DEPTH_SIZE 1] ~width:500 ~height:500 ~packing:w#add () in
+    GlGtk.area [`RGBA;`DEPTH_SIZE 1;`DOUBLEBUFFER]
+      ~width:500 ~height:500 ~packing:w#add () in
   area#connect#realize ~callback:
     begin fun () ->
       GlMat.mode `projection;
@@ -24,10 +25,11 @@ let main () =
       GlDraw.vertex ~x:(0.5) ~y:(0.5) ();
       GlDraw.vertex ~x:(0.5) ~y:(-0.5) ();
       GlDraw.ends ();
-      Gl.flush ()
+      Gl.flush ();
+      area#swap_buffers ()
     end;
   Timeout.add ~ms:10000 ~callback:(fun () -> w#destroy ();false);
   w#show ();
   Main.main ()
 
-let () = if not !Sys.interactive then Printexc.print main ()
+let _ = main ()
