@@ -1,4 +1,4 @@
-(* $Id: buttons.ml,v 1.23 2000/05/22 03:14:55 garrigue Exp $ *)
+(* $Id: buttons.ml,v 1.27 2003/07/09 01:59:40 garrigue Exp $ *)
 
 open GMain
 
@@ -15,11 +15,18 @@ let xpm_label_box ~(window : #GContainer.container)
 let main () =
   let window = GWindow.window ~title:"Pixmap'd Buttons!" ~border_width:10 () in
   window#connect#destroy ~callback:Main.quit;
-  let button = GButton.button ~packing:window#add () in
+  let hbox = GPack.hbox ~packing:window#add () in
+  let button = GButton.button ~packing:(hbox#pack ~padding:5) () in
   button#connect#clicked ~callback:
     (fun () -> prerr_endline "Hello again - cool button was pressed");
   xpm_label_box ~window ~file:"test.xpm" ~text:"cool button"
     ~packing:button#add ();
+  let button = GButton.button ~use_mnemonic:true ~label:"_Coucou" ~packing:(hbox#pack ~padding:5) () in
+  button#connect#clicked ~callback:
+    (fun () -> prerr_endline "Coucou");
+  let button = GButton.button ~stock:`HOME ~packing:(hbox#pack ~padding:5) () in
+  button#connect#clicked ~callback:
+    (fun () -> prerr_endline "Stock buttons look nice");
   window#show ();
   Main.main ()
 
