@@ -1,4 +1,4 @@
-(* $Id: dcalendar.ml,v 1.6 2001/09/06 08:47:55 garrigue Exp $ *)
+(* $Id: dcalendar.ml,v 1.11 2004/11/29 02:18:32 garrigue Exp $ *)
 
 (* A small calendar *)
 (* Needs Unix module, so use with lablgtk_t *)
@@ -14,7 +14,7 @@ let calendar_file = Filename.concat (Sys.getenv "HOME") ".camlendar"
 
 let schedule =
   try
-    let ichan = open_in calendar_file in
+    let ichan = open_in_bin calendar_file in
     let (s : (int * int * int, string) Hashtbl.t) =
       Marshal.from_channel ichan in
     close_in ichan;
@@ -25,7 +25,7 @@ let schedule =
 
     (* Saves the schedule data when the application terminates *)
 at_exit (fun () ->
-  let ochan = open_out calendar_file in
+  let ochan = open_out_bin calendar_file in
   Marshal.to_channel ochan schedule [];
   close_out ochan);;
 
@@ -131,7 +131,7 @@ let create_GUI () =
     GWindow.window ~title: "Camlendar" ~show: true
       ~allow_shrink: false ~allow_grow: false () in
   win#event#connect#delete
-    ~callback: (fun _ -> GMain.Main.quit (); exit 0; false);
+    ~callback: (fun _ -> GMain.Main.quit (); false);
 
   let style = win#misc#style#copy in
   styles.(s_normal) <- style;
@@ -171,12 +171,12 @@ let create_GUI () =
 
   let date_view = GMisc.label ~justify: `CENTER ~packing () in
 
-  let text = GEdit.text ~editable: true ~width: 70 ~height: 50 ~packing () in
+  let text = GText.view ~editable:true ~width:70 ~height:50 ~packing () in
 
   (* Controls part *)
 
   let save_text () =
-    let data = text#get_chars ~start: 0 ~stop: text#length in
+    let data = text#buffer#get_text () in
     let key = (date.year, date.mon, date.mday) in
     Hashtbl.remove schedule key;
     if data <> "" then
@@ -185,12 +185,14 @@ let create_GUI () =
     else buttons.(date.mday - 1)#unset_plan in
 
   let restore_text () =
-    text#delete_text ~start: 0 ~stop: text#length;
     try
-      text#insert_text ~pos: 0
+      text#buffer#set_text
  	(Hashtbl.find schedule (date.year, date.mon, date.mday));
       ()
-    with Not_found -> () in
+    with Not_found -> 
+      let start,stop = text#buffer#bounds in
+      text#buffer#delete ~start ~stop
+  in
 
   let update_date_view () =
     date_view#set_text (sprintf "%d %s, %d\n"
@@ -243,7 +245,7 @@ let create_GUI () =
   buttons.(0)#focus_on;;
 
 GMain.Main.init ();
-print_endline (GtkMain.Main.set_locale ());
+print_endline (Glib.Main.setlocale `ALL None);
 flush stdout;
 create_GUI ();
 GMain.Main.main ()
