@@ -1,4 +1,6 @@
-(* $Id: editor.ml,v 1.22 2001/09/06 08:47:55 garrigue Exp $ *)
+(* $Id: editor.ml,v 1.26 2004/07/01 12:29:50 garrigue Exp $ *)
+
+(* Uses the deprecated old text widget. See editor2.ml for the new one *)
 
 open StdLabels
 open GMain
@@ -9,14 +11,14 @@ let file_dialog ~title ~callback ?filename () =
   sel#cancel_button#connect#clicked ~callback:sel#destroy;
   sel#ok_button#connect#clicked ~callback:
     begin fun () ->
-      let name = sel#get_filename in
+      let name = sel#filename in
       sel#destroy ();
       callback name
     end;
   sel#show ()
 
 class editor ?packing ?show () = object (self)
-  val text = GEdit.text ~editable:true ?packing ?show ()
+  val text = GBroken.text ~editable:true ?packing ?show ()
   val mutable filename = None
 
   method text = text
