@@ -1,26 +1,21 @@
-(* $Id: gData.mli,v 1.12 2001/12/10 06:49:08 garrigue Exp $ *)
+(* $Id: gData.mli,v 1.22 2004/07/05 10:05:47 oandrieu Exp $ *)
 
 open Gtk
 
-class data_signals :
-  'a obj ->
+(** Non-Widgets objects carrying data *)
+
+(** {3 GtkAdjustement} *)
+
+(** @gtkdoc gtk GtkAdjustment *)
+class adjustment_signals : [> adjustment] obj ->
   object
     inherit GObj.gtkobj_signals
-    constraint 'a = [>`data]
-    val obj : 'a obj
-    method disconnect_data : callback:(unit -> unit) -> GtkSignal.id
-  end
-
-class adjustment_signals :
-  'a obj ->
-  object
-    inherit data_signals
-    constraint 'a = [>`adjustment|`data]
-    val obj : 'a obj
     method changed : callback:(unit -> unit) -> GtkSignal.id
     method value_changed : callback:(unit -> unit) -> GtkSignal.id
   end
 
+(** A GtkObject representing an adjustable bounded value
+   @gtkdoc gtk GtkAdjustment *)
 class adjustment : Gtk.adjustment obj ->
   object
     inherit GObj.gtkobj
@@ -39,6 +34,13 @@ class adjustment : Gtk.adjustment obj ->
       ?lower:float -> ?upper:float -> ?step_incr:float ->
       ?page_incr:float -> ?page_size:float -> unit -> unit
   end
+
+(** @gtkdoc gtk GtkAdjustment 
+    @param lower default value is [0.]
+    @param upper default value is [100.]
+    @param step_incr default value is [1.]
+    @param page_incr default value is [10.]
+    @param page_size default value is [10.] *)
 val adjustment :
   ?value:float ->
   ?lower:float ->
@@ -47,17 +49,42 @@ val adjustment :
   ?page_incr:float -> ?page_size:float -> unit -> adjustment
 
 val as_adjustment : adjustment -> Gtk.adjustment obj
+val conv_adjustment : adjustment Gobject.data_conv
+val conv_adjustment_option : adjustment option Gobject.data_conv
 
+(** {3 Tooltips} *)
+
+(** Add tips to your widgets
+   @gtkdoc gtk GtkTooltips *)
 class tooltips :
   Gtk.tooltips obj ->
   object
     inherit GObj.gtkobj
     val obj : Gtk.tooltips obj
     method as_tooltips : Gtk.tooltips obj
-    method connect : data_signals
+    method connect : GObj.gtkobj_signals
     method disable : unit -> unit
     method enable : unit -> unit
     method set_delay : int -> unit
     method set_tip : ?text:string -> ?privat:string -> GObj.widget -> unit
   end
+
+(** @gtkdoc gtk GtkTooltips *)
 val tooltips : ?delay:int -> unit -> tooltips
+
+(** {3 Clipboards} *)
+
+(** Storing data on clipboards
+   @gtkdoc gtk gtk-Clipboards *)
+class clipboard : Gtk.clipboard Lazy.t ->
+  object
+    method as_clipboard : Gtk.clipboard
+    method clear : unit -> unit
+    method get_contents : target:Gdk.atom -> GObj.selection_data
+    method set_text : string -> unit
+    method text : string option
+  end
+
+(** @gtkdoc gtk gtk-Clipboards *)
+val clipboard : Gdk.atom -> clipboard
+val as_clipboard : clipboard -> Gtk.clipboard
