@@ -1,15 +1,16 @@
-(* $Id: gUtil.ml,v 1.17 2001/09/17 13:09:33 garrigue Exp $ *)
+(* $Id: gUtil.ml,v 1.21 2004/12/03 22:55:57 garrigue Exp $ *)
 
 open StdLabels
 open GObj
 
+let print_widget ppf (o : #widget) = 
+    Format.fprintf ppf "<%s@@0x%x>" o#misc#get_type o#get_oid
+
 class ['a] memo () = object
-  constraint 'a = #widget
-  val tbl = Hashtbl.create 7
-  method add (obj : 'a) =
-    Hashtbl.add tbl obj#get_id obj
-  method find (obj : widget) = Hashtbl.find tbl obj#get_id
-  method remove (obj : widget) = Hashtbl.remove tbl obj#get_id
+  val tbl : (int, 'a) Hashtbl.t = Hashtbl.create 7
+  method add (obj : 'a) = Hashtbl.add tbl obj#get_oid obj
+  method find (obj : widget) = Hashtbl.find tbl obj#get_oid
+  method remove (obj : widget) = Hashtbl.remove tbl obj#get_oid
 end
 
 let signal_id = ref 0
