@@ -1,13 +1,24 @@
-(* $Id: gdkPixbuf.ml,v 1.3.2.1 2003/06/11 08:30:05 garrigue Exp $ *)
+(* $Id: gdkPixbuf.ml,v 1.12 2004/08/23 21:37:40 oandrieu Exp $ *)
 
 open Gaux
+open Gobject
 open Gdk
 
-type pixbuf
+type pixbuf = [`pixbuf] obj
 type colorspace = [ `RGB ]
 type alpha_mode = [ `BILEVEL | `FULL ]
 type interpolation = [ `NEAREST | `TILES | `BILINEAR | `HYPER ]
-type uint8 = int
+
+type gdkpixbuferror =
+  | ERROR_CORRUPT_IMAGE
+  | ERROR_INSUFFICIENT_MEMORY
+  | ERROR_BAD_OPTION
+  | ERROR_UNKNOWN_TYPE
+  | ERROR_UNSUPPORTED_OPERATION
+  | ERROR_FAILED
+exception GdkPixbufError of gdkpixbuferror * string
+external _init : unit -> unit = "ml_gdkpixbuf_init"
+let () = _init () ; Callback.register_exception "gdk_pixbuf_error" (GdkPixbufError (ERROR_CORRUPT_IMAGE, ""))
 
 (* Accessors *)
 
@@ -23,7 +34,7 @@ external _get_pixels : pixbuf -> Obj.t * int = "ml_gdk_pixbuf_get_pixels"
 let get_pixels pixbuf =
   let obj, pos = _get_pixels pixbuf in
   let get_length (_, pixbuf) =
-    get_rowstride pixbuf * (get_height pixbuf - 1) + get_width pixbuf + pos
+    get_rowstride pixbuf * get_height pixbuf + pos
   in
   let r =
     Gpointer.unsafe_create_region ~path:[|0|] ~get_length (obj, pixbuf) in
@@ -38,8 +49,14 @@ external _create :
 let create ~width ~height ?(bits=8) ?(colorspace=`RGB) ?(has_alpha=false) () =
   _create ~colorspace ~has_alpha ~bits ~width ~height
 
+let cast o : pixbuf = Gobject.try_cast o "GdkPixbuf"
+
 external copy : pixbuf -> pixbuf = "ml_gdk_pixbuf_copy" 
+external subpixbuf : pixbuf -> src_x:int -> src_y:int -> width:int -> height:int -> pixbuf 
+  = "ml_gdk_pixbuf_new_subpixbuf"
 external from_file : string -> pixbuf = "ml_gdk_pixbuf_new_from_file"
+external from_file_at_size : string -> width:int -> height:int -> pixbuf 
+  = "ml_gdk_pixbuf_new_from_file_at_size"
 external from_xpm_data : string array -> pixbuf
   = "ml_gdk_pixbuf_new_from_xpm_data"
 
@@ -51,17 +68,17 @@ let from_data ~width ~height ?(bits=8) ?rowstride ?(has_alpha=false) data =
   let nc = if has_alpha then 4 else 3 in
   let rowstride = match rowstride with None -> width * nc | Some r -> r in
   if bits <> 8 || rowstride < width * nc || width <= 0 || height <= 0
-  || Gpointer.length data < (rowstride * (height - 1) + width) * nc
+  || Gpointer.length data < rowstride * (height - 1) + width * nc
   then invalid_arg "GdkPixbuf.from_data";
   _from_data data ~has_alpha ~bits ~width ~height ~rowstride
 
 external _get_from_drawable :
-  pixbuf -> 'a drawable -> colormap -> src_x:int -> src_y:int ->
+  pixbuf -> [>`drawable] obj -> colormap -> src_x:int -> src_y:int ->
   dest_x:int -> dest_y:int -> width:int -> height:int -> unit
   = "ml_gdk_pixbuf_get_from_drawable_bc" "ml_gdk_pixbuf_get_from_drawable"
 let get_from_drawable ~dest ?(dest_x=0) ?(dest_y=0) ?width ?height
     ?(src_x=0) ?(src_y=0) ?(colormap=Gdk.Rgb.get_cmap()) src =
-  let dw, dh = Gdk.Window.get_size src in
+  let dw, dh = Gdk.Drawable.get_size src in
   let mw = min (dw - src_x) (get_width dest - dest_x)
   and mh = min (dh - src_y) (get_height dest - dest_y) in
   let width = default mw ~opt:width and height = default mh ~opt:height in
@@ -84,22 +101,25 @@ let render_alpha bm ?(dest_x=0) ?(dest_y=0) ?width ?height ?(threshold=128)
   and height = may_default get_height src ~opt:height in
   _render_alpha ~src bm ~src_x ~src_y ~dest_x ~dest_y ~width ~height ~threshold
 
-external _render_to_drawable :
-  src:pixbuf -> 'a drawable -> gc -> src_x:int -> src_y:int ->
+external _draw_pixbuf :
+  src:pixbuf -> [>`drawable] obj -> gc -> src_x:int -> src_y:int ->
   dest_x:int -> dest_y:int -> width:int -> height:int ->
   dither:Tags.rgb_dither -> x_dither:int -> y_dither:int -> unit
   = "ml_gdk_pixbuf_render_to_drawable_bc"
     "ml_gdk_pixbuf_render_to_drawable"
-let render_to_drawable dw ?(gc=Gdk.GC.create dw) ?(dest_x=0) ?(dest_y=0)
+let draw_pixbuf dw gc ?(dest_x=0) ?(dest_y=0)
     ?width ?height ?(dither=`NONE) ?(x_dither=0) ?(y_dither=0)
     ?(src_x=0) ?(src_y=0) src =
   let width = may_default get_width src ~opt:width
   and height = may_default get_height src ~opt:height in
-  _render_to_drawable ~src dw gc ~src_x ~src_y ~dest_x ~dest_y ~width ~height
+  _draw_pixbuf dw gc ~src ~src_x ~src_y ~dest_x ~dest_y ~width ~height
     ~dither ~x_dither ~y_dither
 
+let render_to_drawable dw ?(gc=Gdk.GC.create dw) =
+  draw_pixbuf dw gc
+
 external _render_to_drawable_alpha :
-  src:pixbuf -> 'a drawable -> src_x:int -> src_y:int ->
+  src:pixbuf -> [>`drawable] obj -> src_x:int -> src_y:int ->
   dest_x:int -> dest_y:int -> width:int -> height:int ->
   alpha:alpha_mode -> threshold:int ->
   dither:Tags.rgb_dither -> x_dither:int -> y_dither:int -> unit
@@ -125,6 +145,12 @@ let add_alpha ?transparent pb =
   match transparent with None -> _add_alpha pb ~subst:false ~r:0 ~g:0 ~b:0
   | Some (r, g, b) -> _add_alpha pb ~subst:true ~r ~g ~b
 
+external fill : pixbuf -> int32 -> unit = "ml_gdk_pixbuf_fill"
+external _saturate_and_pixelate : pixbuf -> dest:pixbuf -> saturation:float -> pixelate:bool -> unit
+    = "ml_gdk_pixbuf_saturate_and_pixelate"
+let saturate_and_pixelate ~dest ~saturation ~pixelate src =
+  _saturate_and_pixelate src ~dest ~saturation ~pixelate
+
 external _copy_area :
   src:pixbuf -> src_x:int -> src_y:int -> width:int -> height:int ->
   dest:pixbuf -> dest_x:int -> dest_y:int -> unit
@@ -180,3 +206,15 @@ let composite ~dest ~alpha ?(dest_x=0) ?(dest_y=0) ?width ?height
   in
   _composite ~src ~dest ~dest_x ~dest_y ~width ~height ~ofs_x ~ofs_y ~scale_x
     ~scale_y ~interp ~alpha
+
+(* Saving *)
+
+external save : filename:string -> typ:string -> ?options:(string * string) list -> pixbuf -> unit
+    = "ml_gdk_pixbuf_save"
+
+external save_to_callback : 
+  pixbuf -> typ:string -> ?options:(string * string) list -> 
+  (string -> unit) -> unit = "ml_gdk_pixbuf_save_to_callback"
+
+let save_to_buffer pb ~typ ?options buffer =
+  save_to_callback pb ~typ ?options (Buffer.add_string buffer)
