@@ -1,114 +1,157 @@
-/* $Id: wrappers.h,v 1.36 2001/05/28 17:57:14 furuse Exp $ */
+/* $Id: wrappers.h,v 1.61 2004/11/10 19:54:03 oandrieu Exp $ */
 
 #ifndef _wrappers_
 #define _wrappers_
 
+/* Yell if a caml callback raised an exception */
+#define CAML_EXN_LOG(name) g_critical("%s: callback raised an exception", name)
+
+#include <caml/misc.h>
 #include <caml/mlvalues.h>
 #include <caml/fail.h>
+#include <caml/custom.h>
+CAMLextern char *young_start, *young_end; /* from minor_gc.h */
 
 value copy_memblock_indirected (void *src, asize_t size);
-value ml_some (value);
+value alloc_memblock_indirected (asize_t size);
+CAMLprim value ml_some (value);
+value ml_cons (value, value);
 void ml_raise_null_pointer (void) Noreturn;
 value Val_pointer (void *);
-value copy_string_check (const char*);
+CAMLprim value copy_string_check (const char*);
 value copy_string_or_null (const char *);
 
-value *ml_global_root_new (value v);
+CAMLprim value *ml_global_root_new (value v);
 void ml_global_root_destroy (void *data);
 
+/* enums <-> polymorphic variants */
 typedef struct { value key; int data; } lookup_info;
-value ml_lookup_from_c (lookup_info *table, int data);
-int ml_lookup_to_c (lookup_info *table, value key);
+value ml_lookup_from_c (const lookup_info table[], int data);
+int ml_lookup_to_c (const lookup_info table[], value key);
+value ml_lookup_flags_getter (const lookup_info table[], int data);
+
+/* Compatibility */
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2,2,0) && !defined(DISABLE_GTK22)
+#define HASGTK22
+#endif
+#if GTK_CHECK_VERSION(2,4,0) && !defined(DISABLE_GTK24)
+#define HASGTK24
+#endif
 
 /* Wrapper generators */
 
+#define Unsupported(cname) \
+CAMLprim value ml_##cname () \
+{ failwith(#cname " unsupported in Gtk 2.x < 2.2"); return Val_unit; }
+#define Unsupported_24(cname) \
+CAMLprim value ml_##cname () \
+{ failwith(#cname " unsupported in Gtk 2.x < 2.4"); return Val_unit; }
+
+#define ID(x) (x)
+
 #define ML_0(cname, conv) \
-value ml_##cname (value unit) { return conv (cname ()); }
+CAMLprim value ml_##cname (value unit) { return conv (cname ()); }
 #define ML_1(cname, conv1, conv) \
-value ml_##cname (value arg1) { return conv (cname (conv1 (arg1))); }
+CAMLprim value ml_##cname (value arg1) { return conv (cname (conv1 (arg1))); }
 #define ML_1_post(cname, conv1, conv, post) \
-value ml_##cname (value arg1) \
+CAMLprim value ml_##cname (value arg1) \
 { value ret = conv (cname (conv1(arg1))); post; return ret; }
 #define ML_2(cname, conv1, conv2, conv) \
-value ml_##cname (value arg1, value arg2) \
+CAMLprim value ml_##cname (value arg1, value arg2) \
 { return conv (cname (conv1(arg1), conv2(arg2))); }
 #define ML_2_name(mlname, cname, conv1, conv2, conv) \
-value mlname (value arg1, value arg2) \
+CAMLprim value mlname (value arg1, value arg2) \
 { return conv (cname (conv1(arg1), conv2(arg2))); }
 #define ML_3(cname, conv1, conv2, conv3, conv) \
-value ml_##cname (value arg1, value arg2, value arg3) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3))); }
 #define ML_3_name(mlname, cname, conv1, conv2, conv3, conv) \
-value mlname (value arg1, value arg2, value arg3) \
+CAMLprim value mlname (value arg1, value arg2, value arg3) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3))); }
 #define ML_4(cname, conv1, conv2, conv3, conv4, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4))); }
 #define ML_4_name(mlname, cname, conv1, conv2, conv3, conv4, conv) \
-value mlname (value arg1, value arg2, value arg3, value arg4) \
+CAMLprim value mlname (value arg1, value arg2, value arg3, value arg4) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4))); }
 #define ML_5(cname, conv1, conv2, conv3, conv4, conv5, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5))); }
 #define ML_5_name(mlname, cname, conv1, conv2, conv3, conv4, conv5, conv) \
-value mlname (value arg1, value arg2, value arg3, value arg4, value arg5) \
+CAMLprim value mlname (value arg1, value arg2, value arg3, value arg4, \
+                       value arg5) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5))); }
 #define ML_6(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6))); }
 #define ML_7(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7))); }
 #define ML_8(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
 	     conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8))); }
 #define ML_9(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
 	      conv9, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8, value arg9) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8, \
+                           value arg9) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
 		      conv9(arg9))); }
-#define ML_9_name(mlname, cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
-	      conv9, conv) \
-value mlname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8, value arg9) \
+#define ML_9_name(mlname, cname, conv1, conv2, conv3, conv4, conv5, conv6, \
+                  conv7, conv8, conv9, conv) \
+CAMLprim value mlname (value arg1, value arg2, value arg3, value arg4, \
+                       value arg5, value arg6, value arg7, value arg8, \
+                       value arg9) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
 		      conv9(arg9))); }
 #define ML_10(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
 	      conv9, conv10, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8, value arg9, value arg10)\
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8, \
+                           value arg9, value arg10)\
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
 		      conv9(arg9), conv10(arg10))); }
 #define ML_11(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
 	      conv9, conv10, conv11, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8, value arg9, value arg10,\
-		  value arg11) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8, \
+                           value arg9, value arg10, value arg11) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
 		      conv9(arg9), conv10(arg10), conv11(arg11))); }
 #define ML_12(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
 	      conv9, conv10, conv11, conv12, conv) \
-value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
-		  value arg6, value arg7, value arg8, value arg9, value arg10,\
-		  value arg11, value arg12) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8, \
+                           value arg9, value arg10, value arg11, value arg12) \
 { return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
 		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
 		      conv9(arg9), conv10(arg10), conv11(arg11), \
 		      conv12(arg12))); }
+#define ML_13(cname, conv1, conv2, conv3, conv4, conv5, conv6, conv7, conv8, \
+	      conv9, conv10, conv11, conv12, conv13, conv) \
+CAMLprim value ml_##cname (value arg1, value arg2, value arg3, value arg4, \
+                           value arg5, value arg6, value arg7, value arg8, \
+                           value arg9, value arg10, value arg11, value arg12, \
+                           value arg13) \
+{ return conv (cname (conv1(arg1), conv2(arg2), conv3(arg3), conv4(arg4), \
+		      conv5(arg5), conv6(arg6), conv7(arg7), conv8(arg8), \
+		      conv9(arg9), conv10(arg10), conv11(arg11), \
+		      conv12(arg12), conv13(arg13))); }
 
 /* Use with care: needs the argument index */
 #define Ignore(x)
@@ -120,31 +163,35 @@ value ml_##cname (value arg1, value arg2, value arg3, value arg4, value arg5, \
 
 /* For more than 5 arguments */
 #define ML_bc6(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5]); }
 #define ML_bc7(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6]); }
 #define ML_bc8(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
 	       argv[7]); }
 #define ML_bc9(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
 	       argv[7],argv[8]); }
 #define ML_bc10(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
 	       argv[7],argv[8],argv[9]); }
 #define ML_bc11(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
 	       argv[7],argv[8],argv[9],argv[10]); }
 #define ML_bc12(cname) \
-value cname##_bc (value *argv, int argn) \
+CAMLprim value cname##_bc (value *argv, int argn) \
 { return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
 	       argv[7],argv[8],argv[9],argv[10],argv[11]); }
+#define ML_bc13(cname) \
+CAMLprim value cname##_bc (value *argv, int argn) \
+{ return cname(argv[0],argv[1],argv[2],argv[3],argv[4],argv[5],argv[6], \
+	       argv[7],argv[8],argv[9],argv[10],argv[11],argv[12]); }
 
 /* result conversion */
 #define Unit(x) ((x), Val_unit)
@@ -155,11 +202,20 @@ value cname##_bc (value *argv, int argn) \
 #define Bool_ptr(x) ((long) x - 1)
 #define Char_val Int_val
 #define Float_val(x) ((float)Double_val(x))
+#define SizedString_val(x) String_val(x), string_length(x)
 
 #define Option_val(val,unwrap,default) \
 ((long)val-1 ? unwrap(Field(val,0)) : default)
 #define String_option_val(s) Option_val(s,String_val,NULL)
 
+/* Strings are not always old, so they may move around... */
+/* problems with alloca on Linux
+#define StableString_val(val) \
+  ((char*)(val) < young_end && (char*)(val) > young_start ? \
+   memcpy(alloca(Bosize_val(val)), (char*)(val), Bosize_val(val)) : \
+   String_val(val))
+*/
+
 /* Utility */
 
 #define Copy_array(ret,l,src,conv) \
@@ -172,21 +228,42 @@ value cname##_bc (value *argv, int argn) \
 #define Make_Val_final_pointer(type, init, final, adv) \
 static void ml_final_##type (value val) \
 { if (Field(val,1)) final ((type*)Field(val,1)); } \
+static struct custom_operations ml_custom_##type = \
+{ #type"/2.0/", ml_final_##type, custom_compare_default, \
+  custom_hash_default, custom_serialize_default, custom_deserialize_default };\
 value Val_##type (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
-  ret = alloc_final (2, ml_final_##type, adv, 1000); \
+  ret = alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
   initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Make_Val_final_pointer_ext(type, ext, init, final, adv) \
 static void ml_final_##type##ext (value val) \
 { if (Field(val,1)) final ((type*)Field(val,1)); } \
+static struct custom_operations ml_custom_##type##ext = \
+{ #type#ext"/2.0/", ml_final_##type##ext, custom_compare_default, \
+  custom_hash_default, custom_serialize_default, custom_deserialize_default };\
 value Val_##type##ext (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
-  ret = alloc_final (2, ml_final_##type##ext, adv, 1000); \
+  ret = alloc_custom (&ml_custom_##type##ext, sizeof(value), adv, 1000); \
+  initialize (&Field(ret,1), (value) p); init(p); return ret; }
+
+#define Make_Val_final_pointer_compare(type, init, comp, final, adv) \
+static void ml_final_##type (value val) \
+{ if (Field(val,1)) final ((type*)Field(val,1)); } \
+static int ml_comp_##type(value v1, value v2) \
+{ return comp((type*)Field(v1,1), (type*)Field(v2,1)); } \
+static struct custom_operations ml_custom_##type = \
+{ #type"/2.0/", ml_final_##type, ml_comp_##type, \
+  custom_hash_default, custom_serialize_default, custom_deserialize_default };\
+value Val_##type (type *p) \
+{ value ret; if (!p) ml_raise_null_pointer(); \
+  ret = alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
   initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Pointer_val(val) ((void*)Field(val,1))
-#define MLPointer_val(val) (Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
+#define Store_pointer(val,p) (Field(val,1)=Val_bp(p))
+#define MLPointer_val(val) \
+        (Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
 
 #define Val_addr(ptr) (1+(value)ptr)
 #define Addr_val(val) ((void*)(val-1))
@@ -195,19 +272,19 @@ value Val_##type##ext (type *p) \
 #define Wosizeof(x) Wosize_asize(sizeof(x))
 
 #define Make_Extractor(name,conv1,field,conv2) \
-value ml_##name##_##field (value val) \
+CAMLprim value ml_##name##_##field (value val) \
 { return conv2 ((conv1(val))->field); }
 
 #define Make_Setter(name,conv1,conv2,field) \
-value ml_##name##_##field (value val, value new) \
+CAMLprim value ml_##name##_##field (value val, value new) \
 { (conv1(val))->field = conv2(new); return Val_unit; }
 
 #define Make_Array_Extractor(name,conv1,conv2,field,conv) \
-value ml_##name##_##field (value val, value index) \
+CAMLprim value ml_##name##_##field (value val, value index) \
 { return conv ((conv1(val))->field[conv2(index)]); }
 
 #define Make_Array_Setter(name,conv1,conv2,conv3,field) \
-value ml_##name##_##field (value val, value index, value new) \
+CAMLprim value ml_##name##_##field (value val, value index, value new) \
 { (conv1(val))->field[conv2(index)] = conv3(new); return Val_unit; }
 
 /* ML value is [flag list] */
@@ -233,4 +310,7 @@ int OptFlags_##conv (value list) \
 
 #define Check_null(v) (v ? v : (ml_raise_null_pointer (), v))
 
+#define Val_nativeint copy_nativeint
+#define Val_int64 copy_int64
+
 #endif /* _wrappers_ */
